@@ -569,6 +569,8 @@ do_cluster_crosscheck(void)
 
 	t_node_status_cube **cube;
 
+	bool		error_found = false;
+
 	n = build_cluster_crosscheck(&cube, &name_length);
 	if (runtime_options.output_mode == OM_CSV)
 	{
@@ -648,9 +650,11 @@ do_cluster_crosscheck(void)
 				{
 					case -2:
 						c = '?';
+						error_found = true;
 						break;
 					case -1:
 						c = 'x';
+						error_found = true;
 						break;
 					case 0:
 						c = '*';
@@ -689,6 +693,11 @@ do_cluster_crosscheck(void)
 
 		free(cube);
 	}
+
+	if (error_found == true)
+	{
+		exit(ERR_CLUSTER_CHECK);
+	}
 }
 
 
@@ -704,6 +713,8 @@ do_cluster_matrix()
 
 	t_node_matrix_rec **matrix_rec_list;
 
+	bool		error_found = false;
+
 	n = build_cluster_matrix(&matrix_rec_list, &name_length);
 
 	if (runtime_options.output_mode == OM_CSV)
@@ -742,9 +753,11 @@ do_cluster_matrix()
 				{
 					case -2:
 						c = '?';
+						error_found = true;
 						break;
 					case -1:
 						c = 'x';
+						error_found = true;
 						break;
 					case 0:
 						c = '*';
@@ -770,6 +783,11 @@ do_cluster_matrix()
 	}
 
 	free(matrix_rec_list);
+
+	if (error_found == true)
+	{
+		exit(ERR_CLUSTER_CHECK);
+	}
 }
 
 
@@ -964,8 +982,7 @@ build_cluster_matrix(t_node_matrix_rec ***matrix_rec_dest, int *name_length)
 
 		initPQExpBuffer(&command_output);
 
-		(void) remote_command(
-							  host,
+		(void) remote_command(host,
 							  runtime_options.remote_user,
 							  command.data,
 							  &command_output);
@@ -1144,9 +1161,8 @@ build_cluster_crosscheck(t_node_status_cube ***dest_cube, int *name_length)
 		/* fix to work with --node-id */
 		if (cube[i]->node_id == config_file_options.node_id)
 		{
-			(void) local_command(
-								 command.data,
-								 &command_output);
+			(void) local_command_simple(command.data,
+										&command_output);
 		}
 		else
 		{
@@ -1170,8 +1186,7 @@ build_cluster_crosscheck(t_node_status_cube ***dest_cube, int *name_length)
 
 			log_verbose(LOG_DEBUG, "build_cluster_crosscheck(): executing\n  %s", quoted_command.data);
 
-			(void) remote_command(
-								  host,
+			(void) remote_command(host,
 								  runtime_options.remote_user,
 								  quoted_command.data,
 								  &command_output);
