@@ -940,6 +940,7 @@ do_node_check_replication_connection(void)
 		return;
 	}
 
+	/* retrieve remote node record from local database */
 	local_conn = establish_db_connection(config_file_options.conninfo, true);
 
 	record_status = get_node_record(local_conn, runtime_options.remote_node_id, &node_record);
@@ -956,8 +957,12 @@ do_node_check_replication_connection(void)
 	initialize_conninfo_params(&remote_conninfo, false);
 	parse_conninfo_string(node_record.conninfo, &remote_conninfo, NULL, false);
 
+	if (strcmp(param_get(&remote_conninfo, "user"), node_record.repluser) != 0)
+	{
+		param_set(&remote_conninfo, "user", node_record.repluser);
+		param_set(&remote_conninfo, "dbname", "replication");
+	}
 	param_set(&remote_conninfo, "replication", "1");
-	param_set(&remote_conninfo, "user", node_record.repluser);
 
 	repl_conn = establish_db_connection_by_params(&remote_conninfo, false);
 
@@ -1513,7 +1518,7 @@ do_node_service(void)
 
 	if (data_dir_required_for_action(action))
 	{
-		get_node_data_directory(data_dir);
+		get_node_config_directory(data_dir);
 
 		if (data_dir[0] == '\0')
 		{
@@ -1601,7 +1606,7 @@ _do_node_service_list_actions(t_server_action action)
 
 	if (data_dir_required == true)
 	{
-		get_node_data_directory(data_dir);
+		get_node_config_directory(data_dir);
 	}
 
 	/* show command for specific action only */
@@ -1667,6 +1672,13 @@ parse_server_action(const char *action_name)
  *
  * Note that "repmgr node rejoin" is also executed by
  * "repmgr standby switchover" after promoting the new primary.
+ *
+ * Parameters:
+ *   --dry-run
+ *   --force-rewind[=VALUE]
+ *   --config-files
+ *   --config-archive-dir
+ *   -W/--no-wait
  */
 void
 do_node_rejoin(void)
@@ -1728,7 +1740,7 @@ do_node_rejoin(void)
 		{
 			log_error(_("database is not shut down cleanly"));
 
-			if (runtime_options.force_rewind == true)
+			if (runtime_options.force_rewind_used == true)
 			{
 				log_detail(_("pg_rewind will not be able to run"));
 			}
@@ -1758,7 +1770,17 @@ do_node_rejoin(void)
 	PQfinish(upstream_conn);
 
 	/* connect to registered primary and check it's not in recovery */
-	upstream_conn = establish_db_connection(primary_node_record.conninfo, true);
+	upstream_conn = establish_db_connection(primary_node_record.conninfo, false);
+
+	if (PQstatus(upstream_conn) != CONNECTION_OK)
+	{
+		log_error(_("unable to connect to current primary \"%s\" (node ID: %i)"),
+				  primary_node_record.node_name,
+				  primary_node_record.node_id);
+		log_detail(_("primay node conninfo is: \"%s\""),
+				   primary_node_record.conninfo);
+		exit(ERR_BAD_CONFIG);
+	}
 
 	upstream_recovery_type = get_recovery_type(upstream_conn);
 
@@ -1774,30 +1796,33 @@ do_node_rejoin(void)
 	}
 
 	/*
-	 * If --force-rewind specified, check pg_rewind can be used, and
-	 * pre-emptively fetch the list of configuration files which should be
-	 * archived
+	 * --force-rewind specified - check prerequisites, and attempt to execute
+  	 * (if --dry-run provided, just output the command which would be executed)
 	 */
 
-	if (runtime_options.force_rewind == true)
+
+	if (runtime_options.force_rewind_used == true)
 	{
-		PQExpBufferData reason;
 		PQExpBufferData msg;
+		PQExpBufferData	filebuf;
+		int				ret;
 
-		initPQExpBuffer(&reason);
+		/*
+		 * Check that pg_rewind can be used
+		 */
 
-		if (can_use_pg_rewind(upstream_conn, config_file_options.data_directory, &reason) == false)
+		initPQExpBuffer(&msg);
+
+		if (can_use_pg_rewind(upstream_conn, config_file_options.data_directory, &msg) == false)
 		{
 			log_error(_("--force-rewind specified but pg_rewind cannot be used"));
-			log_detail("%s", reason.data);
-			termPQExpBuffer(&reason);
+			log_detail("%s", msg.data);
+			termPQExpBuffer(&msg);
 			PQfinish(upstream_conn);
 
 			exit(ERR_BAD_CONFIG);
 		}
-		termPQExpBuffer(&reason);
 
-		initPQExpBuffer(&msg);
 		appendPQExpBuffer(&msg,
 						  _("prerequisites for using pg_rewind are met"));
 
@@ -1810,25 +1835,31 @@ do_node_rejoin(void)
 			log_verbose(LOG_INFO, "%s", msg.data);
 		}
 		termPQExpBuffer(&msg);
-	}
-
-	/*
-	 * Forcibly rewind node if requested (this is mainly for use when this
-	 * action is being executed by "repmgr standby switchover")
-	 */
-	if (runtime_options.force_rewind == true && runtime_options.dry_run == false)
-	{
-		int			ret;
-		PQExpBufferData		filebuf;
 
+		/*
+		 * Archive requested configuration files.
+		 *
+		 * In --dry-run mode this acts as a check that the files can be archived, though
+		 * errors will only be logged; any copied files will be deleted and --dry-run
+		 * execution will continue.
+		 */
 		_do_node_archive_config();
 
 		/* execute pg_rewind */
 		initPQExpBuffer(&command);
 
-		appendPQExpBuffer(&command,
-						  "%s -D ",
-						  make_pg_path("pg_rewind"));
+		if (runtime_options.force_rewind_path[0] != '\0')
+		{
+			appendPQExpBuffer(&command,
+							  "%s -D ",
+							  runtime_options.force_rewind_path);
+		}
+		else
+		{
+			appendPQExpBuffer(&command,
+							  "%s -D ",
+							  make_pg_path("pg_rewind"));
+		}
 
 		appendShellString(&command,
 						  config_file_options.data_directory);
@@ -1842,112 +1873,119 @@ do_node_rejoin(void)
 			log_info(_("pg_rewind would now be executed"));
 			log_detail(_("pg_rewind command is:\n  %s"),
 						 command.data);
-
-			PQfinish(upstream_conn);
-			exit(SUCCESS);
 		}
-
-		log_notice(_("executing pg_rewind"));
-		log_debug("pg_rewind command is:\n  %s",
-				  command.data);
-
-		initPQExpBuffer(&command_output);
-
-		ret = local_command(
-							command.data,
-							&command_output);
-
-		termPQExpBuffer(&command);
-
-		if (ret == false)
+		else
 		{
-			log_error(_("unable to execute pg_rewind"));
-			log_detail("%s", command_output.data);
+			log_notice(_("executing pg_rewind"));
+			log_debug("pg_rewind command is:\n  %s",
+					  command.data);
+
+			initPQExpBuffer(&command_output);
+
+			ret = local_command(command.data,
+								&command_output);
+
+			termPQExpBuffer(&command);
+
+			if (ret == false)
+			{
+				log_error(_("unable to execute pg_rewind"));
+				log_detail("%s", command_output.data);
+
+				termPQExpBuffer(&command_output);
+
+				exit(ERR_BAD_CONFIG);
+			}
 
 			termPQExpBuffer(&command_output);
 
-			exit(ERR_BAD_CONFIG);
-		}
+			/* Restore any previously archived config files */
+			_do_node_restore_config();
 
-		termPQExpBuffer(&command_output);
+			initPQExpBuffer(&filebuf);
 
-		/* Restore any previously archived config files */
-		_do_node_restore_config();
-
-		initPQExpBuffer(&filebuf);
-
-		/* remove any recovery.done file copied in by pg_rewind */
-		appendPQExpBuffer(&filebuf,
-						  "%s/recovery.done",
-						  config_file_options.data_directory);
-
-		if (stat(filebuf.data, &statbuf) == 0)
-		{
-			log_verbose(LOG_INFO, _("deleting \"recovery.done\""));
-
-			if (unlink(filebuf.data) == -1)
-			{
-				log_warning(_("unable to delete \"%s\""),
-							filebuf.data);
-				log_detail("%s", strerror(errno));
-			}
-		}
-		termPQExpBuffer(&filebuf);
-
-		/* delete any replication slots copied in by pg_rewind */
-		{
-			PQExpBufferData slotdir_path;
-			DIR			  *slotdir;
-			struct dirent *slotdir_ent;
-
-			initPQExpBuffer(&slotdir_path);
-
-			appendPQExpBuffer(&slotdir_path,
-							  "%s/pg_replslot",
+			/* remove any recovery.done file copied in by pg_rewind */
+			appendPQExpBuffer(&filebuf,
+							  "%s/recovery.done",
 							  config_file_options.data_directory);
 
-			slotdir = opendir(slotdir_path.data);
-
-			if (slotdir == NULL)
+			if (stat(filebuf.data, &statbuf) == 0)
 			{
-				log_warning(_("unable to open replication slot directory \"%s\""),
-							slotdir_path.data);
-				log_detail("%s", strerror(errno));
-			}
-			else
-			{
-				while ((slotdir_ent = readdir(slotdir)) != NULL) {
-					struct stat statbuf;
-					PQExpBufferData slotdir_ent_path;
+				log_verbose(LOG_INFO, _("deleting \"recovery.done\""));
 
-					if(strcmp(slotdir_ent->d_name, ".") == 0 || strcmp(slotdir_ent->d_name, "..") == 0)
-						continue;
-
-					initPQExpBuffer(&slotdir_ent_path);
-
-					appendPQExpBuffer(&slotdir_ent_path,
-									  "%s/%s",
-									  slotdir_path.data,
-									  slotdir_ent->d_name);
-
-					if (stat(slotdir_ent_path.data, &statbuf) == 0 && !S_ISDIR(statbuf.st_mode))
-					{
-						termPQExpBuffer(&slotdir_ent_path);
-						continue;
-					}
-
-					log_debug("deleting slot directory \"%s\"", slotdir_ent_path.data);
-					if (rmdir_recursive(slotdir_ent_path.data) != 0 && errno != EEXIST)
-					{
-						log_warning(_("unable to delete replication slot directory \"%s\""), slotdir_ent_path.data);
-						log_detail("%s", strerror(errno));
-						log_hint(_("directory may need to be manually removed"));
-					}
-
-					termPQExpBuffer(&slotdir_ent_path);
+				if (unlink(filebuf.data) == -1)
+				{
+					log_warning(_("unable to delete \"%s\""),
+								filebuf.data);
+					log_detail("%s", strerror(errno));
 				}
 			}
-			termPQExpBuffer(&slotdir_path);
+			termPQExpBuffer(&filebuf);
+
+			/*
+			 * Delete any replication slots copied in by pg_rewind.
+			 *
+			 * TODO:
+			 *  - from PostgreSQL 11, this will be handled by pg_rewind, so
+			 *    we can skip this step from that version; see commit
+			 *    266b6acb312fc440c1c1a2036aa9da94916beac6
+			 *  - possibly delete contents of various other directories
+			 *    as per the above commit for pre-PostgreSQL 11
+			 */
+			{
+				PQExpBufferData slotdir_path;
+				DIR			  *slotdir;
+				struct dirent *slotdir_ent;
+
+				initPQExpBuffer(&slotdir_path);
+
+				appendPQExpBuffer(&slotdir_path,
+								  "%s/pg_replslot",
+								  config_file_options.data_directory);
+
+				slotdir = opendir(slotdir_path.data);
+
+				if (slotdir == NULL)
+				{
+					log_warning(_("unable to open replication slot directory \"%s\""),
+								slotdir_path.data);
+					log_detail("%s", strerror(errno));
+				}
+				else
+				{
+					while ((slotdir_ent = readdir(slotdir)) != NULL) {
+						struct stat statbuf;
+						PQExpBufferData slotdir_ent_path;
+
+						if(strcmp(slotdir_ent->d_name, ".") == 0 || strcmp(slotdir_ent->d_name, "..") == 0)
+							continue;
+
+						initPQExpBuffer(&slotdir_ent_path);
+
+						appendPQExpBuffer(&slotdir_ent_path,
+										  "%s/%s",
+										  slotdir_path.data,
+										  slotdir_ent->d_name);
+
+						if (stat(slotdir_ent_path.data, &statbuf) == 0 && !S_ISDIR(statbuf.st_mode))
+						{
+							termPQExpBuffer(&slotdir_ent_path);
+							continue;
+						}
+
+						log_debug("deleting slot directory \"%s\"", slotdir_ent_path.data);
+						if (rmdir_recursive(slotdir_ent_path.data) != 0 && errno != EEXIST)
+						{
+							log_warning(_("unable to delete replication slot directory \"%s\""), slotdir_ent_path.data);
+							log_detail("%s", strerror(errno));
+							log_hint(_("directory may need to be manually removed"));
+						}
+
+						termPQExpBuffer(&slotdir_ent_path);
+					}
+				}
+				termPQExpBuffer(&slotdir_path);
+			}
 		}
 	}
 
@@ -1967,7 +2005,9 @@ do_node_rejoin(void)
 	if (success == false)
 	{
 		log_notice(_("NODE REJOIN failed"));
-		log_detail("%s", follow_output.data);
+
+		if (strlen(follow_output.data))
+			log_detail("%s", follow_output.data);
 
 		create_event_notification(upstream_conn,
 								  &config_file_options,
@@ -1983,22 +2023,99 @@ do_node_rejoin(void)
 	}
 
 	/*
-	 * XXX add checks that node actually started and connected to primary,
-	 * if not exit with ERR_REJOIN_FAIL
+	 * Actively check that node actually started and connected to primary,
+	 * if not exit with ERR_REJOIN_FAIL.
+	 *
+	 * This check can be overridden with -W/--no-wait, in which case a one-time
+	 * check will be carried out.
 	 */
+	if (runtime_options.no_wait == false)
+	{
+		int i;
 
-	create_event_notification(upstream_conn,
-							  &config_file_options,
-							  config_file_options.node_id,
-							  "node_rejoin",
-							  success,
-							  follow_output.data);
+		for (i = 0; i < config_file_options.standby_reconnect_timeout; i++)
+		{
+			if (is_server_available(config_file_options.conninfo))
+			{
+				log_verbose(LOG_INFO, _("demoted primary is pingable"));
+				break;
+			}
 
-	PQfinish(upstream_conn);
+			if (i % 5 == 0)
+			{
+				log_verbose(LOG_INFO, _("waiting for node %i to respond to pings; %i of max %i attempts"),
+							config_file_options.node_id,
+							i + 1, config_file_options.standby_reconnect_timeout);
+			}
+			else
+			{
+				log_debug("sleeping 1 second waiting for node %i to respond to pings; %i of max %i attempts",
+						  config_file_options.node_id,
+						  i + 1, config_file_options.standby_reconnect_timeout);
+			}
 
-	log_notice(_("NODE REJOIN successful"));
-	log_detail("%s", follow_output.data);
+			sleep(1);
+		}
 
+		for (;  i < config_file_options.standby_reconnect_timeout; i++)
+		{
+			success = is_downstream_node_attached(upstream_conn, config_file_options.node_name);
+
+			if (success == true)
+			{
+				log_verbose(LOG_INFO, _("node %i has attached to its upstream node"),
+							config_file_options.node_id);
+				break;
+			}
+
+			if (i % 5 == 0)
+			{
+				log_info(_("waiting for node %i to connect to new primary; %i of max %i attempts"),
+						 config_file_options.node_id,
+						 i + 1, config_file_options.standby_reconnect_timeout);
+			}
+			else
+			{
+				log_debug("sleeping 1 second waiting for node %i to connect to new primary; %i of max %i attempts",
+						  config_file_options.node_id,
+						  i + 1, config_file_options.standby_reconnect_timeout);
+			}
+
+			sleep(1);
+		}
+
+		create_event_notification(upstream_conn,
+								  &config_file_options,
+								  config_file_options.node_id,
+								  "node_rejoin",
+								  success,
+								  follow_output.data);
+
+		if (success == false)
+		{
+			termPQExpBuffer(&follow_output);
+			log_notice(_("NODE REJOIN failed"));
+			exit(ERR_REJOIN_FAIL);
+		}
+	}
+	else
+	{
+		success = is_downstream_node_attached(upstream_conn, config_file_options.node_name);
+	}
+
+
+	if (success == true)
+	{
+		log_notice(_("NODE REJOIN successful"));
+		log_detail("%s", follow_output.data);
+	}
+	else
+	{
+		/*
+		 * if we reach here, no record found in upstream node's pg_stat_replication */
+		log_notice(_("NODE REJOIN has completed but node is not yet reattached to upstream"));
+		log_hint(_("you will need to manually check the node's replication status"));
+	}
 	termPQExpBuffer(&follow_output);
 
 	return;
@@ -2050,6 +2167,11 @@ _do_node_archive_config(void)
 			termPQExpBuffer(&archive_dir);
 			exit(ERR_BAD_CONFIG);
 		}
+
+		if (runtime_options.dry_run == true)
+		{
+			log_verbose(LOG_INFO, "temporary archive directory \"%s\" created", archive_dir.data);
+		}
 	}
 	else if (!S_ISDIR(statbuf.st_mode))
 	{
@@ -2074,8 +2196,8 @@ _do_node_archive_config(void)
 	{
 
 		/*
-		 * attempt to remove any existing files in the directory TODO: collate
-		 * problem files into list
+		 * attempt to remove any existing files in the directory
+		 * TODO: collate problem files into list
 		 */
 		while ((arcdir_ent = readdir(arcdir)) != NULL)
 		{
@@ -2151,7 +2273,11 @@ _do_node_archive_config(void)
 
 		if (i < config_file_len)
 		{
-			strncpy(filenamebuf, runtime_options.config_files + i, config_file_len - i);
+			int			filename_len = config_file_len - i;
+
+			strncpy(filenamebuf, runtime_options.config_files + i, filename_len);
+
+			filenamebuf[filename_len] = '\0';
 
 			initPQExpBuffer(&pathbuf);
 			appendPQExpBuffer(&pathbuf,
@@ -2229,7 +2355,7 @@ _do_node_archive_config(void)
 		}
 		else
 		{
-			log_verbose(LOG_INFO, "directory \"%s\" deleted", archive_dir.data);
+			log_verbose(LOG_INFO, "temporary archive directory \"%s\" deleted", archive_dir.data);
 		}
 	}
 
@@ -2438,15 +2564,15 @@ do_node_help(void)
 	puts("");
 	printf(_("  Configuration file required, runs on local node only.\n"));
 	puts("");
-	printf(_("    --csv                 emit output as CSV\n"));
-	printf(_("    --nagios              emit output in Nagios format (individual status output only)\n"));
+	printf(_("    --csv                   emit output as CSV\n"));
+	printf(_("    --nagios                emit output in Nagios format (individual status output only)\n"));
 	puts("");
 	printf(_("  Following options check an individual status:\n"));
-	printf(_("    --archive-ready       number of WAL files ready for archiving\n"));
-	printf(_("    --downstream          whether all downstream nodes are connected\n"));
-	printf(_("    --replication-lag     replication lag in seconds (standbys only)\n"));
-	printf(_("    --role                check node has expected role\n"));
-	printf(_("    --slots               check for inactive replication slots\n"));
+	printf(_("    --archive-ready         number of WAL files ready for archiving\n"));
+	printf(_("    --downstream            whether all downstream nodes are connected\n"));
+	printf(_("    --replication-lag       replication lag in seconds (standbys only)\n"));
+	printf(_("    --role                  check node has expected role\n"));
+	printf(_("    --slots                 check for inactive replication slots\n"));
 
 	puts("");
 
@@ -2456,13 +2582,16 @@ do_node_help(void)
 	puts("");
 	printf(_("  Configuration file required, runs on local node only.\n"));
 	puts("");
-	printf(_("    --dry-run             check that the prerequisites are met for rejoining the node\n" \
-			 "                          (including usability of \"pg_rewind\" if requested)\n"));
-	printf(_("    --force-rewind        execute \"pg_rewind\" if necessary\n"));
-	printf(_("    --config-files        comma-separated list of configuration files to retain\n" \
-			 "                          after executing \"pg_rewind\"\n"));
-	printf(_("    --config-archive-dir  directory to temporarily store retained configuration files\n" \
-			 "                          (default: /tmp)\n"));
+	printf(_("    --dry-run               check that the prerequisites are met for rejoining the node\n" \
+			 "                              (including usability of \"pg_rewind\" if requested)\n"));
+	printf(_("    --force-rewind[=VALUE]  execute \"pg_rewind\" if necessary\n"));
+	printf(_("                              (9.3 and 9.4 - provide full \"pg_rewind\" path)\n"));
+
+	printf(_("    --config-files          comma-separated list of configuration files to retain\n" \
+			 "                            after executing \"pg_rewind\"\n"));
+	printf(_("    --config-archive-dir    directory to temporarily store retained configuration files\n" \
+			 "                              (default: /tmp)\n"));
+	printf(_("    -W/--no-wait            don't wait for the node to rejoin cluster\n"));
 	puts("");
 
 	printf(_("NODE SERVICE\n"));
