@@ -53,6 +53,7 @@
 
 #include "repmgr.h"
 #include "compat.h"
+#include "controldata.h"
 #include "repmgr-client.h"
 #include "repmgr-client-global.h"
 #include "repmgr-action-primary.h"
@@ -90,6 +91,7 @@ t_node_info target_node_info = T_NODE_INFO_INITIALIZER;
 static ItemList cli_errors = {NULL, NULL};
 static ItemList cli_warnings = {NULL, NULL};
 
+static bool _local_command(const char *command, PQExpBufferData *outputbuf, bool simple);
 
 int
 main(int argc, char **argv)
@@ -176,7 +178,7 @@ main(int argc, char **argv)
 		strncpy(runtime_options.username, pw->pw_name, MAXLEN);
 	}
 
-	while ((c = getopt_long(argc, argv, "?Vb:f:FWd:h:p:U:R:S:D:ck:L:tvC:", long_options,
+	while ((c = getopt_long(argc, argv, "?Vb:f:FwWd:h:p:U:R:S:D:ck:L:tvC:", long_options,
 							&optindex)) != -1)
 	{
 		/*
@@ -241,11 +243,16 @@ main(int argc, char **argv)
 				strncpy(runtime_options.replication_user, optarg, MAXLEN);
 				break;
 
-				/* -W/--wait */
-			case 'W':
+				/* -w/--wait */
+			case 'w':
 				runtime_options.wait = true;
 				break;
 
+				/* -W/--no-wait */
+			case 'W':
+				runtime_options.no_wait = true;
+				break;
+
 				/*----------------------------
 				 * database connection options
 				 *----------------------------
@@ -420,7 +427,13 @@ main(int argc, char **argv)
 				break;
 
 			case OPT_FORCE_REWIND:
-				runtime_options.force_rewind = true;
+				runtime_options.force_rewind_used = true;
+
+				if (optarg != NULL)
+				{
+					strncpy(runtime_options.force_rewind_path, optarg, MAXPGPATH);
+				}
+
 				break;
 
 			case OPT_SIBLINGS_FOLLOW:
@@ -621,7 +634,7 @@ main(int argc, char **argv)
 	 * If -d/--dbname appears to be a conninfo string, validate by attempting
 	 * to parse it (and if successful, store the parsed parameters)
 	 */
-	if (runtime_options.dbname)
+	if (runtime_options.dbname[0])
 	{
 		if (strncmp(runtime_options.dbname, "postgresql://", 13) == 0 ||
 			strncmp(runtime_options.dbname, "postgres://", 11) == 0 ||
@@ -997,7 +1010,6 @@ main(int argc, char **argv)
 		runtime_options.output_mode = OM_OPTFORMAT;
 	}
 
-
 	/*
 	 * Check for configuration file items which can be overriden by runtime
 	 * options
@@ -1055,6 +1067,17 @@ main(int argc, char **argv)
 	if (runtime_options.terse)
 		logger_set_terse();
 
+	/*
+	 * If --dry-run specified, ensure log_level is at least LOG_INFO, regardless
+	 * of what's in the configuration file or -L/--log-level paremeter, otherwise
+	 * some or output might not be displayed.
+	 */
+	if (runtime_options.dry_run == true)
+	{
+		logger_set_min_level(LOG_INFO);
+	}
+
+
 	/*
 	 * Node configuration information is not needed for all actions, with
 	 * STANDBY CLONE being the main exception.
@@ -1315,6 +1338,15 @@ check_cli_parameters(const int action)
 										 _("--no-upstream-connection only effective in Barman mode"));
 					}
 				}
+
+				if (strlen(config_file_options.config_directory))
+				{
+					if (runtime_options.copy_external_config_files == false)
+					{
+						item_list_append(&cli_warnings,
+										 _("\"config_directory\" set in repmgr.conf, but --copy-external-config-files not provided"));
+					}
+				}
 			}
 			break;
 
@@ -1563,6 +1595,41 @@ check_cli_parameters(const int action)
 		}
 	}
 
+	/* --wait/--no-wait */
+
+	if (runtime_options.wait == true && runtime_options.no_wait == true)
+	{
+		item_list_append_format(&cli_errors,
+								_("both --wait and --no-wait options provided"));
+	}
+	else
+	{
+		if (runtime_options.wait)
+		{
+			switch (action)
+			{
+				case STANDBY_FOLLOW:
+					break;
+				default:
+					item_list_append_format(&cli_warnings,
+											_("--wait will be ignored when executing %s"),
+											action_name(action));
+			}
+		}
+		else if (runtime_options.wait)
+		{
+			switch (action)
+			{
+				case NODE_REJOIN:
+					break;
+				default:
+					item_list_append_format(&cli_warnings,
+											_("--no-wait will be ignored when executing %s"),
+											action_name(action));
+			}
+		}
+	}
+
 	/* repmgr node service --action */
 	if (runtime_options.action[0] != '\0')
 	{
@@ -1604,7 +1671,7 @@ check_cli_parameters(const int action)
 		}
 	}
 
-	if (runtime_options.force_rewind == true)
+	if (runtime_options.force_rewind_used == true)
 	{
 		switch (action)
 		{
@@ -2096,12 +2163,28 @@ test_ssh_connection(char *host, char *remote_user)
 }
 
 
+
+
 /*
  * Execute a command locally. "outputbuf" should either be an
  * initialised PQexpbuffer, or NULL
  */
 bool
 local_command(const char *command, PQExpBufferData *outputbuf)
+{
+	return _local_command(command, outputbuf, false);
+}
+
+
+bool
+local_command_simple(const char *command, PQExpBufferData *outputbuf)
+{
+	return _local_command(command, outputbuf, true);
+}
+
+
+static bool
+_local_command(const char *command, PQExpBufferData *outputbuf, bool simple)
 {
 	FILE	   *fp = NULL;
 	char		output[MAXLEN];
@@ -2128,7 +2211,8 @@ local_command(const char *command, PQExpBufferData *outputbuf)
 	while (fgets(output, MAXLEN, fp) != NULL)
 	{
 		appendPQExpBuffer(outputbuf, "%s", output);
-		if (!feof(fp))
+
+		if (!feof(fp) && simple == false)
 		{
 			break;
 		}
@@ -2171,6 +2255,7 @@ get_superuser_connection(PGconn **conn, PGconn **superuser_conn, PGconn **privil
 		log_error(_("no database connection available"));
 		exit(ERR_INTERNAL);
 	}
+
 	is_superuser = is_superuser_connection(*conn, &userinfo);
 
 	if (is_superuser == true)
@@ -2212,6 +2297,8 @@ get_superuser_connection(PGconn **conn, PGconn **superuser_conn, PGconn **privil
 		exit(ERR_BAD_CONFIG);
 	}
 
+	log_debug("established superuser connection as \"%s\"", runtime_options.superuser);
+
 	*privileged_conn = *superuser_conn;
 	return;
 }
@@ -2353,9 +2440,6 @@ copy_remote_files(char *host, char *remote_user, char *remote_path,
 }
 
 
-
-
-
 /*
  * Execute a command via ssh on the remote host.
  *
@@ -2421,7 +2505,7 @@ remote_command(const char *host, const char *user, const char *command, PQExpBuf
 	if (outputbuf != NULL)
 	{
 		if (strlen(outputbuf->data))
-			log_verbose(LOG_DEBUG, "remote_command(): output returned was:\n  %s", outputbuf->data);
+			log_verbose(LOG_DEBUG, "remote_command(): output returned was:\n%s", outputbuf->data);
 		else
 			log_verbose(LOG_DEBUG, "remote_command(): no output returned");
 	}
@@ -2670,6 +2754,33 @@ data_dir_required_for_action(t_server_action action)
 }
 
 
+/*
+ * Copy the location of the configuration file directory into the
+ * provided buffer; if "config_directory" provided, use that, otherwise
+ * default to the data directory.
+ *
+ * This is primarily intended for use with "pg_ctl" (which itself shouldn't
+ * be used outside of development environments).
+ */
+void
+get_node_config_directory(char *config_dir_buf)
+{
+	if (config_file_options.config_directory[0] != '\0')
+	{
+		strncpy(config_dir_buf, config_file_options.config_directory, MAXPGPATH);
+		return;
+	}
+
+	if (config_file_options.data_directory[0] != '\0')
+	{
+		strncpy(config_dir_buf, config_file_options.data_directory, MAXPGPATH);
+		return;
+	}
+
+	return;
+}
+
+
 void
 get_node_data_directory(char *data_dir_buf)
 {
@@ -2734,3 +2845,77 @@ init_node_record(t_node_info *node_record)
 		create_slot_name(node_record->slot_name, config_file_options.node_id);
 	}
 }
+
+
+bool
+can_use_pg_rewind(PGconn *conn, const char *data_directory, PQExpBufferData *reason)
+{
+	bool		can_use = true;
+	int			server_version_num = get_server_version(conn, NULL);
+
+	/* wal_log_hints not available in 9.3, so just determine if data checksums enabled */
+	if (server_version_num < 90400)
+	{
+		int			data_checksum_version = get_data_checksum_version(data_directory);
+
+		if (data_checksum_version < 0)
+		{
+			appendPQExpBuffer(reason,
+							  _("unable to determine data checksum version"));
+			can_use = false;
+		}
+		else if (data_checksum_version == 0)
+		{
+			appendPQExpBuffer(reason,
+							  _("this cluster was initialised without data checksums"));
+			can_use = false;
+		}
+
+		return can_use;
+	}
+
+	/* "full_page_writes" must be on in any case */
+	if (guc_set(conn, "full_page_writes", "=", "off"))
+	{
+		if (can_use == false)
+			appendPQExpBuffer(reason, "; ");
+
+		appendPQExpBuffer(reason,
+						  _("\"full_page_writes\" must be set to \"on\""));
+
+		can_use = false;
+	}
+
+	/*
+	 * "wal_log_hints" off - are data checksums available? Note: we're
+	 * checking the local pg_control file here as the value will be the same
+	 * throughout the cluster and saves a round-trip to the demotion
+	 * candidate.
+	 */
+	if (guc_set(conn, "wal_log_hints", "=", "on") == false)
+	{
+		int			data_checksum_version = get_data_checksum_version(data_directory);
+
+		if (data_checksum_version < 0)
+		{
+			if (can_use == false)
+				appendPQExpBuffer(reason, "; ");
+
+			appendPQExpBuffer(reason,
+							  _("\"wal_log_hints\" is set to \"off\" but unable to determine data checksum version"));
+			can_use = false;
+		}
+		else if (data_checksum_version == 0)
+		{
+			if (can_use == false)
+				appendPQExpBuffer(reason, "; ");
+
+			appendPQExpBuffer(reason,
+							  _("\"wal_log_hints\" is set to \"off\" and data checksums are disabled"));
+
+			can_use = false;
+		}
+	}
+
+	return can_use;
+}
