@@ -35,6 +35,29 @@ do_bdr_node_check(void)
 	/* nothing to do at the moment */
 }
 
+void
+handle_sigint_bdr(SIGNAL_ARGS)
+{
+	PQExpBufferData event_details;
+
+	initPQExpBuffer(&event_details);
+
+	appendPQExpBuffer(&event_details,
+					  "%s signal received",
+					  postgres_signal_arg == SIGTERM
+					  ? "TERM" : "INT");
+
+	create_event_notification(local_conn,
+							  &config_file_options,
+							  config_file_options.node_id,
+							  "repmgrd_shutdown",
+							  true,
+							  event_details.data);
+	termPQExpBuffer(&event_details);
+
+	terminate(SUCCESS);
+}
+
 
 void
 monitor_bdr(void)
@@ -98,23 +121,6 @@ monitor_bdr(void)
 		exit(ERR_BAD_CONFIG);
 	}
 
-	/* Retrieve record for this node from the local database */
-	record_status = get_node_record(local_conn, config_file_options.node_id, &local_node_info);
-
-	/*
-	 * Terminate if we can't find the local node record. This is a
-	 * "fix-the-config" situation, not a lot else we can do.
-	 */
-	if (record_status != RECORD_FOUND)
-	{
-		log_error(_("unable to retrieve record for local node (ID: %i), terminating"),
-				  local_node_info.node_id);
-		log_hint(_("check that \"repmgr bdr register\" was executed for this node"));
-		PQfinish(local_conn);
-		exit(ERR_BAD_CONFIG);
-	}
-
-
 	if (local_node_info.active == false)
 	{
 		log_error(_("local node (ID: %i) is marked as inactive in repmgr"),
@@ -152,15 +158,16 @@ monitor_bdr(void)
 		cell->node_info->node_status = NODE_STATUS_UP;
 	}
 
-	log_debug("main_loop_bdr() monitoring local node %i", config_file_options.node_id);
+	log_info(_("starting continuous BDR node monitoring on node %i"),
+			 config_file_options.node_id);
 
-	log_info(_("starting continuous BDR node monitoring"));
+	INSTR_TIME_SET_CURRENT(log_status_interval_start);
 
 	while (true)
 	{
 
 		/* monitoring loop */
-		log_verbose(LOG_DEBUG, "BDR check loop...");
+		log_verbose(LOG_DEBUG, "BDR check loop - checking %i nodes", nodes.node_count);
 
 		for (cell = nodes.head; cell; cell = cell->next)
 		{
@@ -262,7 +269,6 @@ loop:
 		if (config_file_options.log_status_interval > 0)
 		{
 			int			log_status_interval_elapsed = calculate_elapsed(log_status_interval_start);
-
 			if (log_status_interval_elapsed >= config_file_options.log_status_interval)
 			{
 				log_info(_("monitoring BDR replication status on node \"%s\" (ID: %i)"),
@@ -273,8 +279,7 @@ loop:
 				{
 					if (cell->node_info->monitoring_state == MS_DEGRADED)
 					{
-						log_detail(
-								   _("monitoring node \"%s\" (ID: %i) in degraded mode"),
+						log_detail(_("monitoring node \"%s\" (ID: %i) in degraded mode"),
 								   cell->node_info->node_name,
 								   cell->node_info->node_id);
 					}
