@@ -58,7 +58,7 @@ static FailoverState failover_state = FAILOVER_STATE_UNKNOWN;
 
 static int	primary_node_id = UNKNOWN_NODE_ID;
 static t_node_info upstream_node_info = T_NODE_INFO_INITIALIZER;
-static NodeInfoList standby_nodes = T_NODE_INFO_LIST_INITIALIZER;
+static NodeInfoList sibling_nodes = T_NODE_INFO_LIST_INITIALIZER;
 
 
 static ElectionResult do_election(void);
@@ -74,7 +74,6 @@ static FailoverState follow_new_primary(int new_primary_id);
 static FailoverState witness_follow_new_primary(int new_primary_id);
 
 static void reset_node_voting_status(void);
-void		close_connections_physical();
 
 static bool do_primary_failover(void);
 static bool do_upstream_standby_failover(void);
@@ -142,7 +141,7 @@ do_physical_node_check(void)
 			case FAILOVER_AUTOMATIC:
 				log_error(_("this node is marked as inactive and cannot be used as a failover target"));
 				log_hint(_("%s"), hint);
-				PQfinish(local_conn);
+				close_connection(&local_conn);
 
 				create_event_notification(NULL,
 										  &config_file_options,
@@ -193,7 +192,7 @@ do_physical_node_check(void)
 		if (required_param_missing == true)
 		{
 			log_hint(_("add the missing configuration parameter(s) and start repmgrd again"));
-			PQfinish(local_conn);
+			close_connection(&local_conn);
 			exit(ERR_BAD_CONFIG);
 		}
 	}
@@ -275,7 +274,7 @@ monitor_streaming_primary(void)
 
 				local_node_info.node_status = NODE_STATUS_UNKNOWN;
 
-				PQfinish(local_conn);
+				close_connection(&local_conn);
 
 				/*
 				 * as we're monitoring the primary, no point in trying to
@@ -359,7 +358,7 @@ monitor_streaming_primary(void)
 				if (PQstatus(local_conn) != CONNECTION_OK)
 				{
 					log_warning(_("node appears to be up but no connection could be made"));
-					PQfinish(local_conn);
+					close_connection(&local_conn);
 				}
 				else
 				{
@@ -384,7 +383,7 @@ monitor_streaming_primary(void)
 
 						if (PQstatus(new_primary_conn) != CONNECTION_OK)
 						{
-							PQfinish(new_primary_conn);
+							close_connection(&new_primary_conn);
 							log_warning(_("unable to connect to new primary node %i"), primary_node_id);
 						}
 						else
@@ -450,7 +449,7 @@ monitor_streaming_primary(void)
 
 									termPQExpBuffer(&event_details);
 
-									PQfinish(new_primary_conn);
+									close_connection(&new_primary_conn);
 
 									/* restart monitoring as standby */
 									return;
@@ -468,7 +467,7 @@ monitor_streaming_primary(void)
 								log_error("%s", event_details.data);
 								log_hint(_("check that 'repmgr (primary|standby) register' was executed for this node"));
 
-								PQfinish(new_primary_conn);
+								close_connection(&new_primary_conn);
 
 								create_event_notification(NULL,
 														  &config_file_options,
@@ -540,7 +539,7 @@ loop:
 
 			if (reload_config(&config_file_options))
 			{
-				PQfinish(local_conn);
+				close_connection(&local_conn);
 				local_conn = establish_db_connection(config_file_options.conninfo, true);
 
 				if (*config_file_options.log_file)
@@ -595,7 +594,7 @@ monitor_streaming_standby(void)
 		if (local_node_info.upstream_node_id == NODE_NOT_FOUND)
 		{
 			log_error(_("unable to determine an active primary for this cluster, terminating"));
-			PQfinish(local_conn);
+			close_connection(&local_conn);
 			exit(ERR_BAD_CONFIG);
 		}
 	}
@@ -612,14 +611,14 @@ monitor_streaming_standby(void)
 		log_error(_("no record found for upstream node (ID: %i), terminating"),
 				  local_node_info.upstream_node_id);
 		log_hint(_("ensure the upstream node is registered correctly"));
-		PQfinish(local_conn);
+		close_connection(&local_conn);
 		exit(ERR_DB_CONN);
 	}
 	else if (record_status == RECORD_ERROR)
 	{
 		log_error(_("unable to retrieve record for upstream node (ID: %i), terminating"),
 				  local_node_info.upstream_node_id);
-		PQfinish(local_conn);
+		close_connection(&local_conn);
 		exit(ERR_DB_CONN);
 	}
 
@@ -639,7 +638,7 @@ monitor_streaming_standby(void)
 				  local_node_info.upstream_node_id);
 		log_hint(_("upstream node must be running before repmgrd can start"));
 
-		PQfinish(local_conn);
+		close_connection(&local_conn);
 		exit(ERR_DB_CONN);
 	}
 
@@ -749,7 +748,18 @@ monitor_streaming_standby(void)
 				log_warning("%s", event_details.data);
 				termPQExpBuffer(&event_details);
 
-				PQfinish(upstream_conn);
+				close_connection(&upstream_conn);
+
+				/*
+				 * if local node is unreachable, make a last-minute attempt to reconnect
+				 * before continuing with the failover process
+				 */
+
+				if (PQstatus(local_conn) != CONNECTION_OK)
+				{
+					check_connection(&local_node_info, &local_conn);
+				}
+
 				upstream_conn = try_reconnect(&upstream_node_info);
 
 				/* Node has recovered - log and continue */
@@ -806,6 +816,29 @@ monitor_streaming_standby(void)
 		{
 			int			degraded_monitoring_elapsed = calculate_elapsed(degraded_monitoring_start);
 
+			if (config_file_options.degraded_monitoring_timeout > 0
+				&& degraded_monitoring_elapsed > config_file_options.degraded_monitoring_timeout)
+			{
+				initPQExpBuffer(&event_details);
+
+				appendPQExpBuffer(&event_details,
+								  _("degraded monitoring timeout (%i seconds) exceeded, terminating"),
+								  degraded_monitoring_elapsed);
+
+				log_notice("%s", event_details.data);
+
+				create_event_notification(NULL,
+										  &config_file_options,
+										  config_file_options.node_id,
+										  "repmgrd_shutdown",
+										  true,
+										  event_details.data);
+
+				termPQExpBuffer(&event_details);
+				terminate(ERR_MONITORING_TIMEOUT);
+			}
+
+
 			log_debug("monitoring node %i in degraded state for %i seconds",
 					  upstream_node_info.node_id,
 					  degraded_monitoring_elapsed);
@@ -872,7 +905,49 @@ monitor_streaming_standby(void)
 				/* local node has been promoted */
 				if (get_recovery_type(local_conn) == RECTYPE_PRIMARY)
 				{
-					log_notice(_("local node is primary, checking local node record"));
+					log_notice(_("local node is primary, checking local node state"));
+
+					/*
+					 * It's possible the promote command timed out, but the promotion itself
+					 * succeeded. In this case failover state will be FAILOVER_STATE_PROMOTION_FAILED;
+					 * we can update the node record ourselves and resume primary monitoring.
+					 */
+					if (failover_state == FAILOVER_STATE_PROMOTION_FAILED)
+					{
+						int			degraded_monitoring_elapsed;
+						int			former_upstream_node_id = local_node_info.upstream_node_id;
+
+						update_node_record_set_primary(local_conn,  local_node_info.node_id);
+						record_status = get_node_record(local_conn, local_node_info.node_id, &local_node_info);
+
+						degraded_monitoring_elapsed = calculate_elapsed(degraded_monitoring_start);
+
+						log_notice(_("resuming monitoring as primary node after %i seconds"),
+								   degraded_monitoring_elapsed);
+
+						initPQExpBuffer(&event_details);
+						appendPQExpBuffer(&event_details,
+										  "promotion command failed but promotion completed successfully");
+						create_event_notification(local_conn,
+												  &config_file_options,
+												  local_node_info.node_id,
+												  "repmgrd_failover_promote",
+												  true,
+												  event_details.data);
+
+						termPQExpBuffer(&event_details);
+
+						/* notify former siblings that they should now follow this node */
+						get_active_sibling_node_records(local_conn,
+														local_node_info.node_id,
+														former_upstream_node_id,
+														&sibling_nodes);
+						notify_followers(&sibling_nodes, local_node_info.node_id);
+
+						/* this will restart monitoring in primary mode */
+						monitoring_state = MS_NORMAL;
+						return;
+					}
 
 					/*
 					 * There may be a delay between the node being promoted
@@ -906,12 +981,12 @@ monitor_streaming_standby(void)
 					get_active_sibling_node_records(local_conn,
 													local_node_info.node_id,
 													local_node_info.upstream_node_id,
-													&standby_nodes);
+													&sibling_nodes);
 
-					if (standby_nodes.node_count > 0)
+					if (sibling_nodes.node_count > 0)
 					{
-						log_debug("scanning %i node records to detect new primary...", standby_nodes.node_count);
-						for (cell = standby_nodes.head; cell; cell = cell->next)
+						log_debug("scanning %i node records to detect new primary...", sibling_nodes.node_count);
+						for (cell = sibling_nodes.head; cell; cell = cell->next)
 						{
 							/* skip local node check, we did that above */
 							if (cell->node_info->node_id == local_node_info.node_id)
@@ -930,12 +1005,10 @@ monitor_streaming_standby(void)
 							if (get_recovery_type(cell->node_info->conn) == RECTYPE_PRIMARY)
 							{
 								follow_node_id = cell->node_info->node_id;
-								PQfinish(cell->node_info->conn);
-								cell->node_info->conn = NULL;
+								close_connection(&cell->node_info->conn);
 								break;
 							}
-							PQfinish(cell->node_info->conn);
-							cell->node_info->conn = NULL;
+							close_connection(&cell->node_info->conn);
 						}
 
 						if (follow_node_id != UNKNOWN_NODE_ID)
@@ -943,7 +1016,7 @@ monitor_streaming_standby(void)
 							follow_new_primary(follow_node_id);
 						}
 					}
-					clear_node_info_list(&standby_nodes);
+					clear_node_info_list(&sibling_nodes);
 				}
 			}
 		}
@@ -987,6 +1060,15 @@ loop:
 			}
 		}
 
+		if (PQstatus(primary_conn) == CONNECTION_OK && config_file_options.monitoring_history == true)
+		{
+			update_monitoring_history();
+		}
+		else
+		{
+			connection_ping(local_conn);
+		}
+
 		/*
 		 * handle local node failure
 		 *
@@ -1001,33 +1083,38 @@ loop:
 		{
 			if (local_node_info.active == true)
 			{
+				bool success = true;
+				PQExpBufferData event_details;
+
+				initPQExpBuffer(&event_details);
+
+				local_node_info.active = false;
+
+				appendPQExpBuffer(&event_details,
+								  _("unable to connect to local node \"%s\" (ID: %i), marking inactive"),
+								  local_node_info.node_name,
+								  local_node_info.node_id);
+				log_notice("%s", event_details.data);
+
 				if (PQstatus(primary_conn) == CONNECTION_OK)
 				{
-					if (update_node_record_set_active(primary_conn, local_node_info.node_id, false) == true)
+					if (update_node_record_set_active(primary_conn, local_node_info.node_id, false) == false)
 					{
-						PQExpBufferData event_details;
-
-						initPQExpBuffer(&event_details);
-
-						local_node_info.active = false;
-
-						appendPQExpBuffer(&event_details,
-										  _("unable to connect to local node \"%s\" (ID: %i), marking inactive"),
-										  local_node_info.node_name,
-										  local_node_info.node_id);
-
-						log_warning("%s", event_details.data);
-
-						create_event_notification(primary_conn,
-												  &config_file_options,
-												  local_node_info.node_id,
-												  "standby_failure",
-												  false,
-												  event_details.data);
-
-						termPQExpBuffer(&event_details);
+						success = false;
+						log_warning(_("unable to mark node \"%s\" (ID: %i) as inactive"),
+									  local_node_info.node_name,
+									  local_node_info.node_id);
 					}
 				}
+
+				create_event_notification(primary_conn,
+										  &config_file_options,
+										  local_node_info.node_id,
+										  "standby_failure",
+										  success,
+										  event_details.data);
+
+				termPQExpBuffer(&event_details);
 			}
 		}
 		else
@@ -1066,16 +1153,13 @@ loop:
 		}
 
 
-		if (PQstatus(primary_conn) == CONNECTION_OK && config_file_options.monitoring_history == true)
-			update_monitoring_history();
-
 		if (got_SIGHUP)
 		{
 			log_debug("SIGHUP received");
 
 			if (reload_config(&config_file_options))
 			{
-				PQfinish(local_conn);
+				close_connection(&local_conn);
 				local_conn = establish_db_connection(config_file_options.conninfo, true);
 
 				if (*config_file_options.log_file)
@@ -1125,7 +1209,7 @@ monitor_streaming_witness(void)
 
 		log_error("%s", event_details.data);
 		log_hint(_("execute \"repmgr witness register --force\" to update the witness node "));
-		PQfinish(local_conn);
+		close_connection(&local_conn);
 
 		create_event_notification(NULL,
 								  &config_file_options,
@@ -1153,7 +1237,7 @@ monitor_streaming_witness(void)
 				  upstream_node_info.node_id);
 		log_hint(_("primary node must be running before repmgrd can start"));
 
-		PQfinish(local_conn);
+		close_connection(&local_conn);
 		exit(ERR_DB_CONN);
 	}
 
@@ -1236,7 +1320,7 @@ monitor_streaming_witness(void)
 									true,
 									event_details.data);
 
-				PQfinish(primary_conn);
+				close_connection(&primary_conn);
 				primary_conn = try_reconnect(&upstream_node_info);
 
 				/* Node has recovered - log and continue */
@@ -1334,12 +1418,12 @@ monitor_streaming_witness(void)
 				get_active_sibling_node_records(local_conn,
 												local_node_info.node_id,
 												local_node_info.upstream_node_id,
-												&standby_nodes);
+												&sibling_nodes);
 
-				if (standby_nodes.node_count > 0)
+				if (sibling_nodes.node_count > 0)
 				{
-					log_debug("scanning %i node records to detect new primary...", standby_nodes.node_count);
-					for (cell = standby_nodes.head; cell; cell = cell->next)
+					log_debug("scanning %i node records to detect new primary...", sibling_nodes.node_count);
+					for (cell = sibling_nodes.head; cell; cell = cell->next)
 					{
 						/* skip local node check, we did that above */
 						if (cell->node_info->node_id == local_node_info.node_id)
@@ -1358,12 +1442,10 @@ monitor_streaming_witness(void)
 						if (get_recovery_type(cell->node_info->conn) == RECTYPE_PRIMARY)
 						{
 							follow_node_id = cell->node_info->node_id;
-							PQfinish(cell->node_info->conn);
-							cell->node_info->conn = NULL;
+							close_connection(&cell->node_info->conn);
 							break;
 						}
-						PQfinish(cell->node_info->conn);
-						cell->node_info->conn = NULL;
+						close_connection(&cell->node_info->conn);
 					}
 
 					if (follow_node_id != UNKNOWN_NODE_ID)
@@ -1371,7 +1453,7 @@ monitor_streaming_witness(void)
 						witness_follow_new_primary(follow_node_id);
 					}
 				}
-				clear_node_info_list(&standby_nodes);
+				clear_node_info_list(&sibling_nodes);
 			}
 		}
 loop:
@@ -1425,7 +1507,7 @@ loop:
 
 			if (reload_config(&config_file_options))
 			{
-				PQfinish(local_conn);
+				close_connection(&local_conn);
 				local_conn = establish_db_connection(config_file_options.conninfo, true);
 
 				if (*config_file_options.log_file)
@@ -1472,7 +1554,7 @@ do_primary_failover(void)
 	}
 	else if (election_result == ELECTION_WON)
 	{
-		if (standby_nodes.node_count > 0)
+		if (sibling_nodes.node_count > 0)
 		{
 			log_notice("this node is the winner, will now promote itself and inform other nodes");
 		}
@@ -1517,7 +1599,7 @@ do_primary_failover(void)
 				get_active_sibling_node_records(local_conn,
 												local_node_info.node_id,
 												upstream_node_info.node_id,
-												&standby_nodes);
+												&sibling_nodes);
 
 			}
 			else if (config_file_options.failover == FAILOVER_MANUAL)
@@ -1555,7 +1637,7 @@ do_primary_failover(void)
 											   */
 											  true,
 											  event_details.data);
-					PQfinish(new_primary_conn);
+					close_connection(&new_primary_conn);
 					termPQExpBuffer(&event_details);
 
 				}
@@ -1579,10 +1661,10 @@ do_primary_failover(void)
 	{
 		case FAILOVER_STATE_PROMOTED:
 			/* notify former siblings that they should now follow this node */
-			notify_followers(&standby_nodes, local_node_info.node_id);
+			notify_followers(&sibling_nodes, local_node_info.node_id);
 
 			/* we no longer care about our former siblings */
-			clear_node_info_list(&standby_nodes);
+			clear_node_info_list(&sibling_nodes);
 
 			/* pass control back down to start_monitoring() */
 			log_info(_("switching to primary monitoring mode"));
@@ -1596,10 +1678,10 @@ do_primary_failover(void)
 			 * notify siblings that they should resume following the original
 			 * primary
 			 */
-			notify_followers(&standby_nodes, upstream_node_info.node_id);
+			notify_followers(&sibling_nodes, upstream_node_info.node_id);
 
 			/* we no longer care about our former siblings */
-			clear_node_info_list(&standby_nodes);
+			clear_node_info_list(&sibling_nodes);
 
 			/* pass control back down to start_monitoring() */
 			log_info(_("resuming standby monitoring mode"));
@@ -1774,8 +1856,7 @@ do_upstream_standby_failover(void)
 	int			i, r;
 	char		parsed_follow_command[MAXPGPATH] = "";
 
-	PQfinish(upstream_conn);
-	upstream_conn = NULL;
+	close_connection(&upstream_conn);
 
 	if (get_primary_node_record(local_conn, &primary_node_info) == false)
 	{
@@ -1796,7 +1877,7 @@ do_upstream_standby_failover(void)
 				  primary_node_info.node_name,
 				  primary_node_info.node_id);
 
-		PQfinish(primary_conn);
+		close_connection(&primary_conn);
 		monitoring_state = MS_DEGRADED;
 		INSTR_TIME_SET_CURRENT(degraded_monitoring_start);
 		return false;
@@ -1810,15 +1891,14 @@ do_upstream_standby_failover(void)
 				  primary_node_info.node_name,
 				  primary_node_info.node_id);
 
-		PQfinish(primary_conn);
+		close_connection(&primary_conn);
 		monitoring_state = MS_DEGRADED;
 		INSTR_TIME_SET_CURRENT(degraded_monitoring_start);
 		return false;
 	}
 
 	/* Close the connection to this server */
-	PQfinish(local_conn);
-	local_conn = NULL;
+	close_connection(&local_conn);
 
 	initPQExpBuffer(&event_details);
 
@@ -2137,7 +2217,7 @@ wait_primary_notification(int *new_primary_id)
 			return true;
 		}
 
-		log_verbose(LOG_DEBUG, "waiting for new primary notification, %i of max %i seconds",
+		log_verbose(LOG_DEBUG, "waiting for new primary notification, %i of max %i seconds (\"primary_notification_timeout\")",
 					i, config_file_options.primary_notification_timeout);
 
 		sleep(1);
@@ -2206,7 +2286,7 @@ follow_new_primary(int new_primary_id)
 		{
 			new_primary_ok = false;
 			log_warning(_("new primary is not in recovery"));
-			PQfinish(upstream_conn);
+			close_connection(&upstream_conn);
 		}
 	}
 
@@ -2220,8 +2300,7 @@ follow_new_primary(int new_primary_id)
 	 * restart
 	 */
 
-	PQfinish(local_conn);
-	local_conn = NULL;
+	close_connection(&local_conn);
 
 	/*
 	 * replace %n in "config_file_options.follow_command" with ID of primary
@@ -2240,32 +2319,46 @@ follow_new_primary(int new_primary_id)
 		PGconn	   *old_primary_conn;
 
 		/*
-		 * The follow action could still fail due to the original primary
+		 * The "standby follow" command could still fail due to the original primary
 		 * reappearing before the candidate could promote itself ("repmgr
 		 * standby follow" will refuse to promote another node if the primary
-		 * is available). However the new primary will only instruct use to
-		 * follow it after it's successfully promoted itself, so that very
-		 * likely won't be the reason for the failure.
-		 *
-		 *
-		 * TODO: check the new primary too - we could have a split-brain
-		 * situation where the old primary reappeared just after the new one
-		 * promoted itself.
+		 * is available). However the new primary will only instruct the other
+		 * nodes to follow it after it's successfully promoted itself, so this
+		 * case is highly unlikely. A slightly more likely scenario would
+		 * be the new primary becoming unavailable just after it's sent notifications
+		 * to its follower nodes, and the old primary becoming available again.
 		 */
 		old_primary_conn = establish_db_connection(failed_primary.conninfo, false);
 
 		if (PQstatus(old_primary_conn) == CONNECTION_OK)
 		{
-			/* XXX add event notifications */
 			RecoveryType upstream_recovery_type = get_recovery_type(old_primary_conn);
 
-			PQfinish(old_primary_conn);
-
 			if (upstream_recovery_type == RECTYPE_PRIMARY)
 			{
-				log_notice(_("original primary reappeared - no action taken"));
+				initPQExpBuffer(&event_details);
+				appendPQExpBuffer(&event_details,
+								  _("original primary reappeared - no action taken"));
+
+				log_notice("%s", event_details.data);
+
+				create_event_notification(old_primary_conn,
+										  &config_file_options,
+										  local_node_info.node_id,
+										  "repmgrd_failover_aborted",
+										  true,
+										  event_details.data);
+
+				termPQExpBuffer(&event_details);
+
+				close_connection(&old_primary_conn);
+
 				return FAILOVER_STATE_PRIMARY_REAPPEARED;
 			}
+
+			log_notice(_("original primary reappeared as standby"));
+
+			close_connection(&old_primary_conn);
 		}
 
 		return FAILOVER_STATE_FOLLOW_FAIL;
@@ -2381,7 +2474,7 @@ witness_follow_new_primary(int new_primary_id)
 		{
 			new_primary_ok = false;
 			log_warning(_("new primary is not in recovery"));
-			PQfinish(upstream_conn);
+			close_connection(&upstream_conn);
 		}
 	}
 
@@ -2473,6 +2566,7 @@ do_election(void)
 
 	/* we're visible */
 	int			visible_nodes = 1;
+	int			total_nodes = 0;
 
 	NodeInfoListCell *cell = NULL;
 
@@ -2523,14 +2617,16 @@ do_election(void)
 	get_active_sibling_node_records(local_conn,
 									local_node_info.node_id,
 									upstream_node_info.node_id,
-									&standby_nodes);
+									&sibling_nodes);
+
+	total_nodes = sibling_nodes.node_count + 1;
 
 	log_debug("do_election(): primary location is %s", upstream_node_info.location);
 
 	local_node_info.last_wal_receive_lsn = InvalidXLogRecPtr;
 
 	/* fast path if no other standbys (or witness) exists - normally win by default */
-	if (standby_nodes.node_count == 0)
+	if (sibling_nodes.node_count == 0)
 	{
 		if (strncmp(upstream_node_info.location, local_node_info.location, MAXLEN) == 0)
 		{
@@ -2556,6 +2652,14 @@ do_election(void)
 			return ELECTION_NOT_CANDIDATE;
 		}
 	}
+	else
+	{
+		/* standby nodes found - check if we're in the primary location before checking theirs */
+		if (strncmp(upstream_node_info.location, local_node_info.location, MAXLEN) == 0)
+		{
+			primary_location_seen = true;
+		}
+	}
 
 	/* get our lsn */
 	local_node_info.last_wal_receive_lsn = get_last_wal_receive_location(local_conn);
@@ -2565,7 +2669,7 @@ do_election(void)
 	/* pointer to "winning" node, initially self */
 	candidate_node = &local_node_info;
 
-	for (cell = standby_nodes.head; cell; cell = cell->next)
+	for (cell = sibling_nodes.head; cell; cell = cell->next)
 	{
 		/* assume the worst case */
 		cell->node_info->node_status = NODE_STATUS_UNKNOWN;
@@ -2620,7 +2724,7 @@ do_election(void)
 			candidate_node = cell->node_info;
 		}
 		/* LSN is same - tiebreak on priority, then node_id */
-		else if(cell->node_info->last_wal_receive_lsn == candidate_node->last_wal_receive_lsn)
+		else if (cell->node_info->last_wal_receive_lsn == candidate_node->last_wal_receive_lsn)
 		{
 			log_verbose(LOG_DEBUG, "node %i has same LSN as current candidate %i",
 						cell->node_info->node_id,
@@ -2672,9 +2776,9 @@ do_election(void)
 
 	log_debug("visible nodes: %i; total nodes: %i",
 			  visible_nodes,
-			  standby_nodes.node_count);
+			  total_nodes);
 
-	if (visible_nodes <= (standby_nodes.node_count / 2.0))
+	if (visible_nodes <= (total_nodes / 2.0))
 	{
 		log_notice(_("unable to reach a qualified majority of nodes"));
 		log_detail(_("node will enter degraded monitoring state waiting for reconnect"));
@@ -2853,7 +2957,7 @@ format_failover_state(FailoverState failover_state)
 		case FAILOVER_STATE_FOLLOW_FAIL:
 			return "FOLLOW_FAIL";
 		case FAILOVER_STATE_NODE_NOTIFICATION_ERROR:
-			return "ODE_NOTIFICATION_ERROR";
+			return "NODE_NOTIFICATION_ERROR";
 	}
 
 	/* should never reach here */
@@ -2861,22 +2965,3 @@ format_failover_state(FailoverState failover_state)
 }
 
 
-void
-close_connections_physical()
-{
-	if (PQstatus(primary_conn) == CONNECTION_OK)
-	{
-		/* cancel any pending queries to the primary */
-		if (PQisBusy(primary_conn) == 1)
-			cancel_query(primary_conn, config_file_options.async_query_timeout);
-		PQfinish(primary_conn);
-		primary_conn = NULL;
-	}
-
-	if (upstream_conn != NULL && PQstatus(upstream_conn) == CONNECTION_OK)
-	{
-		PQfinish(upstream_conn);
-		upstream_conn = NULL;
-	}
-
-}
