@@ -53,9 +53,6 @@ bool		startup_event_logged = false;
 MonitoringState monitoring_state = MS_NORMAL;
 instr_time	degraded_monitoring_start;
 
-static void close_connections(void);
-void		(*_close_connections) (void) = NULL;
-
 /*
  * Record receipt of SIGHUP; will cause configuration file to be reread
  * at the appropriate point in the main loop.
@@ -330,7 +327,7 @@ main(int argc, char **argv)
 		{
 			log_error(_("unable to determine status of \"repmgr\" extension"));
 			log_detail("%s", PQerrorMessage(local_conn));
-			PQfinish(local_conn);
+			close_connection(&local_conn);
 			exit(ERR_DB_QUERY);
 		}
 
@@ -347,7 +344,7 @@ main(int argc, char **argv)
 		}
 
 		log_hint(_("check that this node is part of a repmgr cluster"));
-		PQfinish(local_conn);
+		close_connection(&local_conn);
 		exit(ERR_BAD_CONFIG);
 	}
 
@@ -373,7 +370,7 @@ main(int argc, char **argv)
 				break;
 		}
 
-		PQfinish(local_conn);
+		close_connection(&local_conn);
 		terminate(ERR_BAD_CONFIG);
 	}
 
@@ -392,7 +389,7 @@ main(int argc, char **argv)
 		{
 			log_error(_("unable to write to shared memory"));
 			log_hint(_("ensure \"shared_preload_libraries\" includes \"repmgr\""));
-			PQfinish(local_conn);
+			close_connection(&local_conn);
 			terminate(ERR_BAD_CONFIG);
 		}
 	}
@@ -404,7 +401,6 @@ main(int argc, char **argv)
 	}
 	else
 	{
-		_close_connections = close_connections_physical;
 		log_debug("node id is %i, upstream node id is %i",
 				  local_node_info.node_id,
 				  local_node_info.upstream_node_id);
@@ -705,17 +701,29 @@ PGconn *
 try_reconnect(t_node_info *node_info)
 {
 	PGconn	   *conn;
+	t_conninfo_param_list conninfo_params = T_CONNINFO_PARAM_LIST_INITIALIZER;
 
 	int			i;
 
 	int			max_attempts = config_file_options.reconnect_attempts;
 
+	initialize_conninfo_params(&conninfo_params, false);
+
+
+	/* we assume by now the conninfo string is parseable */
+	(void) parse_conninfo_string(node_info->conninfo, &conninfo_params, NULL, false);
+
+	/* set some default values if not explicitly provided */
+	param_set_ine(&conninfo_params, "connect_timeout", "2");
+	param_set_ine(&conninfo_params, "fallback_application_name", "repmgr");
+
 	for (i = 0; i < max_attempts; i++)
 	{
 		log_info(_("checking state of node %i, %i of %i attempts"),
 				 node_info->node_id, i + 1, max_attempts);
-		if (is_server_available(node_info->conninfo) == true)
+		if (is_server_available_params(&conninfo_params) == true)
 		{
+
 			log_notice(_("node has recovered, reconnecting"));
 
 			/*
@@ -723,14 +731,18 @@ try_reconnect(t_node_info *node_info)
 			 * connection denied due to connection exhaustion - fall back to
 			 * degraded monitoring? - make that configurable
 			 */
-			conn = establish_db_connection(node_info->conninfo, false);
+
+			conn = establish_db_connection_by_params(&conninfo_params, false);
+
 			if (PQstatus(conn) == CONNECTION_OK)
 			{
+				free_conninfo_params(&conninfo_params);
+
 				node_info->node_status = NODE_STATUS_UP;
 				return conn;
 			}
 
-			PQfinish(conn);
+			close_connection(&conn);
 			log_notice(_("unable to reconnect to node"));
 		}
 
@@ -742,13 +754,14 @@ try_reconnect(t_node_info *node_info)
 		}
 	}
 
-
 	log_warning(_("unable to reconnect to node %i after %i attempts"),
 				node_info->node_id,
 				max_attempts);
 
 	node_info->node_status = NODE_STATUS_DOWN;
 
+	free_conninfo_params(&conninfo_params);
+
 	return NULL;
 }
 
@@ -784,24 +797,9 @@ print_monitoring_state(MonitoringState monitoring_state)
 }
 
 
-static void
-close_connections()
-{
-	if (_close_connections != NULL)
-		_close_connections();
-
-	if (local_conn != NULL && PQstatus(local_conn) == CONNECTION_OK)
-	{
-		PQfinish(local_conn);
-		local_conn = NULL;
-	}
-
-}
-
 void
 terminate(int retval)
 {
-	close_connections();
 	logger_shutdown();
 
 	if (pid_file)
